package org.one.stone.soup.dfml;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.one.stone.soup.dfml.template.DfmlControllerTemplate;
import org.one.stone.soup.dfml.template.DfmlControllerTemplateController;
import org.one.stone.soup.dfml.template.DfmlStubTemplate;
import org.one.stone.soup.dfml.template.DfmlStubTemplateController;
import org.one.stone.soup.dfml.template.DfmlTemplate;
import org.one.stone.soup.dfml.template.DfmlTemplateController;
import org.one.stone.soup.stringhelper.StringGenerator;
import org.one.stone.soup.xml.XmlElement;

public class DfmlGenerator implements DfmlStubTemplateController, DfmlControllerTemplateController, DfmlTemplateController {

	private DfmlDataDefinition dfml;

	private XmlElement currentTag;
	private String currentAttributeName;
	private String currentAttributeValue;
	
	private int methodTagCursor;
	private int commandTagCursor;
	
	private int methodCursor;
	private int commandCursor;
	private int dataCursor;

	private int commandNumber;
public DfmlGenerator(DfmlDataDefinition dfml) {
	super();

	this.dfml=dfml;
}
/**
 * 
 */
public void generate(boolean generateInterface,boolean generateTemplate,boolean generateStub) throws IOException{

	PrintWriter iOut=null; //interface stream
	PrintWriter dOut=null; //data class stream
	PrintWriter sOut=null; //stub interface implementation stream

	if(dfml.flavour == DfmlDataDefinition.FILE_FLAVOUR)
	{
		if(generateInterface)
		{
			dfml.targetInterfaceFile = dfml.className+"Controller.java";
			iOut = new PrintWriter(
					new FileOutputStream(dfml.targetPath+dfml.targetInterfaceFile)
				);
		}
		if(generateTemplate)
		{
			dfml.targetTemplateFile = dfml.className+".java";
			dOut = new PrintWriter(
					new FileOutputStream(dfml.targetPath+dfml.targetTemplateFile)
				);
		}
		if(generateStub)
		{
			dfml.targetStubFile = dfml.className+"Stub.java";
			sOut = new PrintWriter(
					new FileOutputStream(dfml.targetPath+dfml.targetStubFile)
				);
		}
	}
	else
	{	
		if(generateInterface)
		{
			dfml.targetInterfaceData.setName(dfml.className+"Controller.java");
			iOut = new PrintWriter( dfml.targetInterfaceData );
		}
		
		if(generateTemplate)
		{
			dfml.targetTemplateData.setName(dfml.className+".java");
			dOut = new PrintWriter( dfml.targetTemplateData );
		}
		
		if(generateStub)
		{
			dfml.targetStubData.setName(dfml.className+"Stub.java");
			sOut = new PrintWriter( dfml.targetStubData );
		}
	}
	
	if(generateTemplate)
	{
		DfmlTemplate template = new DfmlTemplate(this);
		while(!template.isFinished())
		{
			dOut.print(template.process());
		}
	}

	if(generateStub)
	{
		DfmlStubTemplate templateS = new DfmlStubTemplate(this);
		while(!templateS.isFinished())
		{
			sOut.print(templateS.process());
		}
	}

	if(generateInterface)
	{
		DfmlControllerTemplate  templateC = new DfmlControllerTemplate (this);
		while(!templateC.isFinished())
		{
			iOut.print(templateC.process());
		}
	}

	try{
		if(generateTemplate)
		{
			dOut.flush();
			dOut.close();
		}

		if(generateInterface)
		{
			iOut.flush();
			iOut.close();
		}

		if(generateStub)
		{
			sOut.flush();
			sOut.close();
		}
	}
	catch(Exception e){}	
}
/**
 * getCallMethodName method comment.
 */
public String getCallMethodName() {
	return (String)dfml.callFns.elementAt(methodCursor);
}
/**
 * getCallMethodNumber method comment.
 */
public String getCallMethodNumber() {
	return ""+methodCursor;
}
/**
 * getStateMethodName method comment.
 */
public String getClauseMethodName() {
	return (String)dfml.clauseFns.elementAt(methodCursor);
}
/**
 * getStateMethodNumber method comment.
 */
public String getClauseMethodNumber() {
	return ""+methodCursor;
}
/**
 * getCommand method comment.
 */
public String getCommand() {
	
	return currentAttributeName;
}
/**
 * getCommandAttribute method comment.
 */
public String getCommandAttribute() {

	return currentAttributeValue;	
}
/**
 * getCommand method comment.
 */
public String getCommandCode() {
	
	return ""+DfmlCommandSet.command(currentAttributeName);
}
/**
 * getCommand method comment.
 */
public String getCommandNumber() {
	
	return ""+commandNumber;
}
/**
 * getData method comment.
 */
public String getData() {
	return "\""+StringGenerator.convertToJavaCodeString(
					(String)dfml.data.elementAt(dataCursor)
				)+"\"";
}
/**
 * getName method comment.
 */
public String getName() {
	return dfml.className;
}
/**
 * getPackgeName method comment.
 */
public String getPackgeName() {
	return dfml.packageName;
}
/**
 * getSourceFile method comment.
 */
public String getSourceFile() {
	return dfml.sourceFile;
}
/**
 * getTimeStamp method comment.
 */
public String getTimeStamp() {

	SimpleDateFormat formatter = new SimpleDateFormat ("yyyy.mm.dd 'at' hh:mm:ss a zzz");
	Date currentTime = new Date();
	return formatter.format(currentTime);
}
/**
 * hasMoreCallMethods method comment.
 */
public boolean hasMoreCallMethods() {

	if(methodCursor<dfml.callFns.size())
		return true;
		
	return false;
}
/**
 * hasMoreStateMethods method comment.
 */
public boolean hasMoreClauseMethods() {
	if(methodCursor<dfml.clauseFns.size())
		return true;
	return false;
}
/**
 * hasMoreCommands method comment.
 */
public boolean hasMoreCommands() {

	commandNumber++;
	if(commandCursor<currentTag.getAttributeCount())
	{
		currentAttributeName = currentTag.getAttributeNameByIndex(commandCursor);
		currentAttributeValue = currentTag.getAttributeValueByIndex(commandCursor);
		return true;
	}
	else
	{
		commandTagCursor++;
		if(commandTagCursor<dfml.tags.size())
		{
			currentTag = (XmlElement)dfml.tags.elementAt(commandTagCursor);

			commandCursor=0;
			if(currentTag.getAttributeCount()==0)
				return false;
			
			currentAttributeName = currentTag.getAttributeNameByIndex(commandCursor);
			currentAttributeValue = currentTag.getAttributeValueByIndex(commandCursor);

			return true;
		}
	}
	
	return false;
}
/**
 * hasMoreData method comment.
 */
public boolean hasMoreData() {

	if(dataCursor<dfml.data.size())
		return true;
	return false;
}
/**
 * initCallMethods method comment.
 */
public String initCallMethods() {
	methodCursor=0;
	
	return null;
}
/**
 * initStateMethods method comment.
 */
public String initClauseMethods() {
	methodCursor=0;
	return null;
}
/**
 * initCommands method comment.
 */
public String initCommands() {
	commandTagCursor=0;
	commandCursor=0;
	commandNumber=0;

	currentTag = (XmlElement)dfml.tags.elementAt(commandTagCursor);
	
	return null;
}
/**
 * initData method comment.
 */
public String initData() {
	dataCursor=0;
	return null;
}
/**
 * nextCallMethod method comment.
 */
public String nextCallMethod() {
	methodCursor++;
	return null;
}
/**
 * nextStateMethod method comment.
 */
public String nextClauseMethod() {
	methodCursor++;
	return null;
}
/**
 * nextCommand method comment.
 */
public String nextCommand() {
	commandCursor++;
	return null;
}
/**
 * nextData method comment.
 */
public String nextData() {
	dataCursor++;
	return null;
}
}
