package org.one.stone.soup.mjdb.data.group;

import java.util.*;

import org.one.stone.soup.mjdb.data.field.*;
import org.one.stone.soup.mjdb.data.functions.*;


/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

public class Row extends Field implements DataGroupInterface,FieldListener{
	private int noOfFields;
	private Vector fields = new Vector();



public Row(int noFields) {
	this(noFields,true);
}
public Row(int noFields,boolean isNull) {
	super();
	Object blankRow;

	noOfFields=noFields;

	for(int loop=0;loop<noOfFields;loop++)
	{
		if(isNull==true)
			blankRow=null;
		else
			blankRow="";
		fields.addElement(blankRow);
	}
}
public void addField(Object data,Object source) throws LockException,SetException {

	if(locked)
		throw new LockException(this+"is locked");

	noOfFields++;

	fields.addElement(data);

	if(report)
		reportChange(source);
}
public void addField(boolean isNull,Object source) throws LockException,SetException {

	if(locked)
		throw new LockException(this+"is locked");

	Object blankRow;

	noOfFields++;

	if(isNull==true)
		blankRow=null;
	else
		blankRow="";
	fields.addElement(blankRow);

	if(report)
		reportChange(source);
}
public int compare(Field target) throws CompareException {
	return 0;
}
public Object getField(int fieldNo) {

	if(fieldNo>=noOfFields)
		return null;

	return fields.elementAt(fieldNo);
}
public int getNoFields() {
	return noOfFields;
}
/**
 * getValue method comment.
 */
public String getValue(String name) {
	return null;
}
public void removeField(int index,Object source) throws LockException {

	if(locked)
		throw new LockException(this+"is locked");

	noOfFields--;

	fields.removeElementAt(index);

	if(report)
		reportChange(source);
}
public void setField(int fieldNo,Object data,Object source) throws LockException,SetException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(fieldNo>=noOfFields)
		throw new SetException("Col:"+fieldNo+" out of range");

	fields.setElementAt(data,fieldNo);

	if(report)
		reportChange(source);
}
/**
 * setValue method comment.
 */
public void setValue(String name, String value, Object source) throws LockException, ConstructException {}

public void autoWrap(Object source) throws SetException,LockException{
	if(locked)
		throw new LockException(this+"is locked");

	for(int loopC=0;loopC<noOfFields;loopC++)
	{
		Object data = Wrapper.wrap( getField(loopC) );

		setField(loopC,data,this);
	}

	if(report)
		reportChange(source);
}

public void autoWrapAsStrings(Object source) throws SetException,LockException{
	if(locked)
		throw new LockException(this+"is locked");

	for(int loopC=0;loopC<noOfFields;loopC++)
	{
		Object data = new StringField( ""+getField(loopC) );

		setField(loopC,data,this);
	}

	if(report)
		reportChange(source);
}
public void addListener(FieldListener listener)
{
	super.addListener(listener);
	for(int loop=0;loop<getNoFields();loop++)
	{
		if( getField(loop) instanceof Field )
		{
			((Field)getField(loop)).addListener(this);
		}
	}
}
/* (non-Javadoc)
 * @see org.one.stone.soup.mjdb.data.field.FieldListener#receiveChange(java.lang.Object, java.lang.Object)
 */
public void receiveChange(Object source, Object data)
{
	reportChange(source);
}

}
