package org.one.stone.soup.xapp.controller.xml;

import java.io.File;

import org.one.stone.soup.xapp.XApplication;
import org.one.stone.soup.xapp.XappReceiver;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.components.tree.XTree;
import org.one.stone.soup.xapp.containers.XappPopup;
import org.one.stone.soup.xapp.containers.XappScrollPane;
import org.one.stone.soup.xapp.filebrowser.XappFileBrowser;
import org.one.stone.soup.xapp.helper.XappWorkingFileManager;
import org.one.stone.soup.xapp.xpath.XappXPathDB;
import org.one.stone.soup.xapp.xpath.XappXPathDB.XappSubXPathDB;
import org.one.stone.soup.xml.XmlElement;

public abstract class XmlApplication extends XApplication implements XappReceiver {

	private XmlApplicationController controller;
	//private String currentFile = "*";
	private XappWorkingFileManager workingFileManager;
	private XappWorkingFileManager schemaFileManager;
	private XappWorkingFileManager schemaViewFileManager;
	
	private String viewName;
	private String formName;
	private String schemaFile;
	
	public XmlApplication(String xappDefinitionFile) throws Exception
	{
		super(xappDefinitionFile);
		
		workingFileManager = new XappWorkingFileManager(null,"untitled","Xml File","xml");
		schemaFileManager = new XappWorkingFileManager(null,"untitled","Schema File","xml");
		schemaViewFileManager = new XappWorkingFileManager(null,"untitled-view","Schema View File","xml");
		
		XTree tree = buildTree();		

		initialise();
		
		XForm form = controller.getFormView();

		((XappScrollPane)XappRootApplication.getComponent(viewName+"Pane")).add(tree);
		((XappScrollPane)XappRootApplication.getComponent(formName+"Pane")).add(form);
			
		getFrame().show();
		
		controller.getTreeView().refreshView();
	}
		
	private XTree buildTree()
	{
		XappSubXPathDB xTreeSchemaDB = getXAppConfig().getSubXPathDB("xapp/application/xTree/schema");
		XappSubXPathDB xTreeViewDB = getXAppConfig().getSubXPathDB("xapp/application/xTree/view");
		viewName = xTreeViewDB.getRoot().getAttributeValueByName("name");		
		XmlElement xDefaultTree = getXAppConfig().findElement("xapp/application/xTree/tree");
		controller = new XmlApplicationController( new File(xTreeViewDB.getRootPath()),xTreeSchemaDB,xDefaultTree.getElementByIndex(0),xTreeViewDB,this );
		
		XmlElement xTree = getXAppConfig().findElement("xapp/application/xTree");
		viewName = xTree.getAttributeValueByName("view");
		formName = xTree.getAttributeValueByName("form");
		
		XTree tree = controller.getTreeView();
		if(viewName!=null)
		{
			setContainer(viewName,tree);
		}
		return tree;
	}

	public void reinitialise()
	{
		XmlElement model = getXmlData();
		XmlElement context = getCurrentSelectedNode();
		String schema = schemaFile;
		
		super.reinitialise();
		
		XTree tree = buildTree();
		setXmlData( model );
		controller.setContext(context);
		openSchema(schemaFile,false);
		
		XForm form = controller.getFormView();

		((XappScrollPane)XappRootApplication.getComponent(viewName)).add(tree);
		((XappScrollPane)XappRootApplication.getComponent(formName)).add(form);
	}
	
	public void processDropAction(String file, int posX, int posY) {
		// TODO Auto-generated method stub
		
	}

	public void processPost(XmlElement data) {
		data = data.getElementByName("data");
		if(data!=null && data.getAttributeValueByName("name").equals("xmlNodeForm"))
		{
			controller.updateNode( data );
			openForm(controller.getModelRoot(),data);
		}
	}

	public void processPopupAction(XmlElement data)
	{
		controller.processPopupAction(data.getElementByIndex(0));
	}
	
	public void processTreeAction( XmlElement dataRoot,XmlElement dataSource,boolean rightButton,int posX,int posY )
	{
		controller.setContext(dataSource);
		
		//if(!rightButton)
		//{
			openForm(dataRoot,dataSource);
		//	return;
		//}
		
		if(rightButton)
		{
			XappPopup popup = controller.getPopup();
			popup.show(getFrame(),posX,posY+50);
		}
	}

	public static XmlApplication getRoot()
	{
		return (XmlApplication)XappRootApplication.getRoot();
	}
	
	public static void openForm(XmlElement root,XmlElement target)
	{
		getRoot()._openForm(root,target);
	}
	
	public void _openForm(XmlElement root,XmlElement target)
	{
		((XappScrollPane)XappRootApplication.getComponent(formName+"Pane")).add(controller.getFormView());
		
		getFrame().refresh();
	}
	
	public void setXmlData(XmlElement xData)
	{
		controller.setModelRoot(xData);
	}
	
	public static XmlElement getXmlData()
	{
		return getRoot().controller.getModelRoot();
	}
	
	public void setSchema(File root,XappSubXPathDB schemaDB,XappSubXPathDB schemaViewDB,boolean recreateDocument)
	{
		controller.setSchema(root,schemaDB,schemaViewDB,recreateDocument);
	}
	
	public boolean open()
	{
		XmlElement newData = workingFileManager.openFileAsXml();
		if(newData!=null)
		{
			setXmlData(newData);
			return true;
		}
		return false;
	}
	
	public void openSchema()
	{
		String file = schemaFileManager.getFileToOpen();
		if(file!=null)
		{
			openSchema(file,true);
		}
	}
	
	public void openSchema(String schemaFile,boolean recreateDocument)
	{
		XappFileBrowser browser = XappRootApplication.getComponentFactory().buildFileBrowser(" schema view file","Xml Schema","xml");
		browser.transferDirectory(" schema file");
			
		try{
			XappXPathDB schemaDB = new XappXPathDB(schemaFile);

			String file = schemaFile.substring(0,schemaFile.length()-4);
			file = file+"-view.xml";
			if( !new File(file).exists() )
			{
				file = schemaViewFileManager.getFileToOpen();
			}
			else
			{
				schemaViewFileManager.setWorkingFileName(file);
			}
			
			if(file!=null)
			{
				XappXPathDB schemaViewDB = new XappXPathDB(file);
				
				setSchema(new File(file).getParentFile(),schemaDB.getSubXPathDB(""),schemaViewDB.getSubXPathDB(""),recreateDocument);
				
				this.schemaFile = schemaFile;
			}
		}
		catch(Exception e)
		{
			XappRootApplication.displayException(e);
		}
	}
	
	public static boolean save()
	{
		return getRoot()._save();
	}
	
	public boolean _save()
	{
		return workingFileManager.save( getXmlData() );
	}
	
	public boolean isSaveFileSet()
	{
		return workingFileManager.isWorkingFileSet();
	}

	public String getCurrentWorkingFileName()
	{
		return workingFileManager.getCurrentFile();
	}
	
	public void clearCurrentFile()
	{
		workingFileManager.clearWorkingFileName();
	}
	
	public boolean saveAs()
	{
		if( workingFileManager.saveAs()==null )
		{
			return false;
		}
		else
		{
			workingFileManager.save( getXmlData() );
			return true;
		}
	}
	
	public static boolean autoSaveIsOn()
	{
		if(getRoot().isSaveFileSet()==false)
		{
			getRoot().getFrame().setTitle( getRoot().getTitle()+" - * Unsaved changes" );
			return false;
		}
		else
		{
			boolean state = XappRootApplication.autoSaveIsOn();
			if(state==false)
			{
				getRoot().getFrame().setTitle( getRoot().getTitle()+" - * Unsaved changes" );
			}
			return state;
		}
	}

	public String getSchemaFile() {
		return schemaFile;
	}
	
	public XmlElement getCurrentSelectedNode()
	{
		return controller.getContext();
	}
}
