package org.one.stone.soup.xapp.swing.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;

import org.one.stone.soup.grfx.ScreenCapture;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.swing.JRootFrame;
import org.one.stone.soup.xapp.XappApplicationFrame;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.XappTheme;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.containers.XappContainer;
import org.one.stone.soup.xapp.containers.XappMenuBar;
import org.one.stone.soup.xapp.templates.XappSchemeToXappThemeConverter;
import org.one.stone.soup.xml.XmlElement;

public class XappSwingApplicationFrame extends JRootFrame implements XappApplicationFrame {

	private ScreenCapture recorder;
	
	public XappSwingApplicationFrame()
	{
		super("untitled",new String[]{});
		getContentPane().setLayout( new BorderLayout() );
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.xapp.XappApplicationFrameInterface#destroy()
	 */
	public boolean destroy() {
		return XappRootApplication.getRoot().destroy();
	}

	public void setRecorder(ScreenCapture recorder)
	{
		this.recorder = recorder;
	}
	
	public void paint(Graphics grfx)
	{
		if(recorder==null)
		{
			super.paint(grfx);
			return;
		}
		
		grfx = recorder.prePaint(grfx,getWidth(),getHeight());
		super.paint(grfx);
		recorder.postPaint(this);
	}
	
	public XappMenuBar getXappMenuBar()
	{
		return (XappMenuBar)getJMenuBar();
	}

	public void setBackgroundColor(Color color) {
		getContentPane().setBackground(color);
	}

	public void setForegroundColor(Color color) {
		getContentPane().setForeground(color);
	}

	public void setXappMenuBar(XappMenuBar menuBar) {
		setJMenuBar( (JMenuBar)menuBar );
	}
	
	public void setContainer(XappContainer container)
	{
		while(getContentPane().getComponentCount()>0)
		{
			getContentPane().remove(0);
		}
		getContentPane().add( (Container)container,BorderLayout.CENTER );
	}

	public boolean askForConfirmation(String message) {
		String[] messageLines = StringArrayHelper.parseFields(message,'\n');

		if( JOptionPane.showConfirmDialog(this,messageLines)==JOptionPane.YES_OPTION )
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	public void showError(String message) {
		String[] messageLines = StringArrayHelper.parseFields(message,'\n');

		JOptionPane.showMessageDialog(this,messageLines,"Error",JOptionPane.ERROR_MESSAGE);
	}

	public void showMessage(String message) {
		String[] messageLines = StringArrayHelper.parseFields(message,'\n');

		JOptionPane.showMessageDialog(this,messageLines,"",JOptionPane.INFORMATION_MESSAGE);
	}

	public String askForValue(String query, String value) {
		return JOptionPane.showInputDialog(this,query,value);
	}

	public String askForSelection(String query,String title, String[] options, String value) {

		return (String)JOptionPane.showInputDialog( this,query,title,JOptionPane.INFORMATION_MESSAGE,null,options,value);
	}
	
	public void refresh()
	{
		invalidate();
		doLayout();
	}
	
	public String getValue()
	{
		return null;
	}

	public void addActionListener(ActionListener actionListener) {
		// TODO Auto-generated method stub
		
	}
	

	public void setTip(String text)
	{
	}

	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void add(String name, XappComponent component) {
		add(name,(Component)component);
	}

	public void add(XappComponent component) {
		add((Component)component);
	}
	
	public void setTheme(XmlElement xTheme)
	{
		try{
			if(xTheme.getName().equals("xScheme"))
			{
				xTheme = XappSchemeToXappThemeConverter.convert(xTheme);
			}
			
			XappTheme theme = new XappTheme(xTheme);
			MetalLookAndFeel laf = new MetalLookAndFeel();
			MetalLookAndFeel.setCurrentTheme(theme);
			UIManager.setLookAndFeel( laf );
		}
		catch(Exception e)
		{
			XappRootApplication.displayException(e);
		}
	}
}
