package org.one.stone.soup.xapp.timing;

import org.one.stone.soup.constants.TimeConstants;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.stringhelper.StringGenerator;
import org.one.stone.soup.xapp.XappRootApplication;

public class StopWatch implements Runnable
{
	private long startTime;
	private long expectedTime;
	private boolean running = false;
	private boolean stopped = true;
	
	private String timeElapsedField;
	private String timeExpectedField;
	private String percentageField;
	
	public void start(String timeElapsedField,String timeExpectedField,String percentageField)
	{
		this.timeElapsedField = timeElapsedField;
		this.timeExpectedField = timeExpectedField;
		this.percentageField = percentageField;		
		
		stop(false);
		
		this.expectedTime = convertToTime( XappRootApplication.getAPI().getField(timeExpectedField) );
		startTime = System.currentTimeMillis();
		
		new Thread(this,"Stop Watch").start();
	}
	
	public void stop(boolean ok)
	{
		running = false;
		while(stopped==false)
		{
			try{ Thread.sleep(1000); }catch(Exception e){}
		}
		
		if(ok)
		{
			XappRootApplication.getAPI().setField( percentageField,"100" );
			XappRootApplication.getAPI().setField( timeExpectedField,this.toString() );
		}
		else
		{
			XappRootApplication.getAPI().setField( percentageField,"0");
		}
	}
	
	public long getElapsedTime()
	{
		return System.currentTimeMillis() - startTime;
	}
	
	public void run()
	{
		running = true;
		stopped = false;
		
		while(running)
		{
			XappRootApplication.getAPI().setField( timeElapsedField,this.toString() );
			XappRootApplication.getAPI().setField( percentageField,""+(getElapsedTime()*100/expectedTime) );
			try{ Thread.sleep(1000); }catch(Exception e){}
		}

		XappRootApplication.getAPI().setField( timeElapsedField,this.toString() );
		
		running = false;
		stopped = true;
	}
	
	public String toString()
	{
		long time = getElapsedTime();
		
		long hours = time/TimeConstants.HOUR_MILLIS;
		time = time-(TimeConstants.HOUR_MILLIS*hours);
		long minutes = time/TimeConstants.MINUTE_MILLIS;
		time = time-(TimeConstants.MINUTE_MILLIS*minutes);
		long seconds = time/1000;
		time = time-(seconds*1000);
		
		String hoursString = ""+hours;
		hoursString = StringGenerator.pad(hoursString,2,'0')+hoursString;
		String minutesString = ""+minutes;
		minutesString = StringGenerator.pad(minutesString,2,'0')+minutesString;
		String secondsString = ""+seconds;
		String millisecondsString = ""+time;
		secondsString = StringGenerator.pad(secondsString,2,'0')+secondsString+"."+StringGenerator.pad(millisecondsString,3,'0')+millisecondsString;
		
		return hoursString+":"+minutesString+":"+secondsString;
	}
	
	public long convertToTime( String time )
	{
		if(time==null || time.length()==0)
		{
			return 1;
		}
		
		String[] parts = StringArrayHelper.parseFields(time,':');
		long millis = Long.parseLong( parts[0] )*TimeConstants.HOUR_MILLIS;
		millis += Long.parseLong( parts[1] )*TimeConstants.MINUTE_MILLIS;
		millis += Double.parseDouble( parts[2] )*1000;
		
		return millis+1;
	}
}

